; Multi-IO V0.11
; (c)W.Rmer 88-2017
;
; Bearbeitung:	19.12.2017	V0.11 mit 4800bd OK
;		19.12.2017	IRQ-Fehler gefunden,  
;				RTS Setzen mit call zerro statt
;				Jump Zerro aufgerufen = Stackfehler
; 		16.12.2017	IRQ-Teil Fehleranalyse
;		15.12.2017	IRQ-Teil Fehleranalyse
;		13.12.2017	V24 mit IRQ
;		05.11.2017	RTC OK
;		02.11.2017	RTC Schreibroutine ueber Buffer
;		01.11.2017	RTC-Routinen ueberarbeitet
;		02.05.2017	RTC-Test	
;		01.05.2017	RTC-Schaltung berichtigt
;		26.04.2017	Uhr schreiben/lesen
;		08.03.2017	Centronics
;		05.03.2017	com1/2 OK
;		03.03.2017	IO-byte OK
;		02.03.2017	Hardware neu aufgebaut
;		08.02.2017	com1,com2 nicht stabil ?
;		06.02.2017	Softwaregrundfunktionen
;				IO-Byte OK
;				Modusumschaltung OK
;				Centronics OK
;				paralelle Ausgabe OK
;		xx.04.1988	Hardware erstellt
;
;====================================================================
;
; V24 mit IRQ Routine - Test fuer Datenempfang
;
; Daten werden von Interupt-Serviceroutine im FIFO abgelegt
; und per Kopierroutine im Zielbuffer gespeichert.
; Wenn der Fuellstand ca. 64% erreicht, wird RTS gesetzt.
; Damit stellt die ISR sicher, dass die Gegenstelle ihren FIFO
; noch leeren kann. 
; Anmerkung! 
; Moderne PC's sprudeln noch bis zu 16Bytes
; aus der V24 obwohl RTS gesetzt ist. 
; Auch bei abgeschalteten FIFO senden sie noch ein bis zwei Bytes
; 
; 
; Parameter zur Ubersicht
;
; buffer	equ 9000h	; fuer BIN-Datei
; zbuff		equ 8400h	; Software FIFO
;
;            v-Schreibzeiger (wrz)
;            I
; 8400h I------------...------------------+-...--I 843fh (maxbuf)
;         ^                               :
;         I- Lesezeiger (rdz)       842ah +------ RTS Setzen (rtsbuf)
;
; maxbuf	equ 3fh		; Bufferlaenge 
; rtsbuf	equ 2ah		; Bufferposition fuer RTS
;
; buffz		dw	0000h	; Schreibzeiger fuer Zielbuffer
; wrz		dw	0000h	; Schreibzeiger fuer IRQ
; rdz		dw	0000h	; Lesezeiger	
; v24flag	db	00h	; 1000 0001
;				; :       :
;				; :       +-- RTS punkt erreicht
;				; +---------- Buffer voll
;
;====================================================================
;
; 2xV24
; 1xCentronics
; 1xparallele Ein/Ausgabe
; 
; Baudrate wird mit Timer 8254 erzeugt
; - com1 300Bd - 9600Bd TX/RX
; - com2 300Bd - 9600Bd TX/RX 
; 
; IO-Byte fuer die Konfiguration
;
;--------------------------------------------------------------------------
; Multi-IO-Schnitstellenkarte
; Portbeschreibung: COM1
;
; Controlregister UART (aus Datenblatt Motorola M6850)
;
;	7 6 5 4 3 2 1 0
;       : : : : : : : :
;       : : : : : : 0 0 = clk/1
;	: : : : : : 0 1 = clk/16
;	: : : : : : 1 0 = clk/64
;	: : : : : : 1 1 = Reset
;       : : : 0 0 0     = ev 2Stop 7Datenbits
;       : : : 0 0 1     = od
;       : : : 0 1 0     = ev 1Stop 	(gerade Paritaet)
;       : : : 0 1 1     = od		(ungerade Paritaet)
;       : : : 1 0 0     = -- 2Stop 8Datenbits
;       : : : 1 0 1     = -- 1Stop
;       : : : 1 1 0     = ev 1 Stop
;       : : : 1 1 1     = od
;	: 0 0           = RTS low 	TX-IRQ disable 
;	: 0 1           = RTS low 	TX-IRQ enable
;	: 1 0           = RTS high 	TX-IRQ disable
;	: 1 1           = RTS low	TX-IRQ disable
;       1               = RX-IRQ enable
;
;	0 0 0 0 0 0 1 1 = 03h Baustein Reset 
;       0 0 0 1 0 1 0 1 = 15h clk/16, 8,n,1
;       0 1 0 1 0 1 0 1 = 55h ... mit RTS=1
;
;	1 0 0 1 0 1 0 1 = 95h clk/16, 8,n,1	; RTS=0 RX-IRQ enable
;	1 1 0 1 0 1 0 1 = D5h      ""		; RTS=1    ""
;
;
; Statusregister UART nur lesen
;
;	7 6 5 4 3 2 1 0
;       : : : : : : : :
;       : : : : : : : 1 = Empfangsregister voll
;	: : : : : : 1   = Senderegister leer
;	: : : : : 1     = DCD Eingang
;	: : : : x       = CTS Eingang
;       : : : 1         = Fehlerbits 	-Frame
;       : : 1           = 		-Overrun
;       : 1             = 		-Paritaet
;       1               = IRQ-Ausgang	(ungerade Paritaet)
;
;--------------------------------------------------------------------
;
; PIO1 Centronics und UNI-IO Kanal
;
pio1a	equ	03h	; Centronic's
pio1b	equ	13h	; frei
pio1c	equ	23h	; Steuerung Centronics
			; 	Bit 7 6 5 4 3 2 1 0
			;	    I I I I I I + + frei
			;	    I I I I I I
			;	    I I I I I + - - /Error	<--
			;	    I I I I + - - - INTRa	-->
			;	    I I I + - - - - Papierende	<--
			;	    I I + - - - - - BUSY	<--
			;	    I + - - - - - - /ACK	<--
			;	    + - - - - - - - /STROBE	-->		
						
pio1s	equ	33h	; Steuerregister
bap1	equ	0abh	; 1010 1011
			; Kanal A Modus 1 Ausgabe, Kanal B Modus 0 Eingabe
			; Kanal C Eingabe
;
; PIO2 IO-Byte und Systemuhr
;
pio2a	equ	43h	; 
			; RTC 58321 Daten
			; 	Bit 7 6 5 4 3 2 1 0
			;	    x x x x + + + + Daten 0-3	<->
			; Bit 4-7 frei
			;	      
pio2b	equ	53h	; IO-byte DIP-Schalter			<--		
			;
pio2c	equ	63h	; RTC 58321 Control
			; 	Bit 7 6 5 4 3 2 1 0
			;	    x x x I I I	I I
			;	    x x x I I I I + WR		-->
			;	    x x x I I I + - RD		-->
			;	    x x x I I + - - AD/WR	-->
			;	    x x x I + - - - CS		-->
			;	    x x x + - - - - Stop	-->
			; Bit 5-7 frei				-->
			;			
pio2s	equ	73h	; Steuerregister
			; Betriebsarten
bap2	equ	82h	; 1000 0010
			; Kanal A Modus 0 Ausgabe 
			; Kanal B Modus 0 Eingabe 
			; Kanal C lo Ausgabe
			; Kanal C hi Ausgabe
			;
bap2rdu	equ	92h	; 1001 0010
			; Kanal A Modus 0 Eingabe 
			; Kanal B Modus 0 Eingabe 
			; Kanal C lo Ausgabe
			; Kanal C hi Ausgabe
;
;--------------------------------------------------------------------
; Baudraten
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;
; fuer 6,144Mhz
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	   01h highByte
;--------------------------------------------------------------------

tim0	equ	83h	; BD-Rate fuer com1
tim1	equ	93h	; BD-Rate fuer com2 
tim2	equ	0a3h	; frei
tims	equ	0b3h	; Steuerregister
			; Timer0-2 als BCD-Zaehler, Rechteckgenerator,
			; erst Timewert-Low, dann Timewert-High laden
bati0	equ	37h	; 0011 0111 
bati1	equ	77h	; 0111 0111
bati2	equ	0b7h	; 1011 0111
;
com1s	equ	0c3h	; Steuerregister 
com1d	equ	0d3h	; Daten com1
bacom1	equ	15h	; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)		;
;
com2s	equ	0e3h	; Steuerregister
com2d	equ	0f3h	; Daten com2
bacom2	equ	15h	; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
;
;
;-----------------------------------------------------------------------------
;
; Vorspann Parameterbereich
;
; Monitorprogramme  
;  
CI	EQU 0043h 	; CI 	Tastaturabfrage wartet auf Taste ASCII(A)    
CO	EQU 0049h  	; CO 	Bildschirmausgabe ASCII(C) Ziel (UPDATE)
LOM	EQU 004Fh	; LO 	Drucker Ueber RS232	          
CSTS	EQU 0052h  	; CSTS	Tastaturstatus Taste Z=0, k.Taste Z=1  	          
FILL	EQU 0085h  	; FILL  Speicher mit Konstante fuellen SRCBEG,SRCEND,DSTBEG        
COPYM	EQU 0088h  	; COPY  Speicherbereich verschieben 
onesec	EQU 001bh	; Wartezeit 1Sekunde
dely1	EQU 000bh	;           1ms
;
BELPOR	EQU 52h		; Port fuer Summer 
BYTFF	EQU 1Ah		; PORT FF auf Singel-Step-Modul
COMREG	EQU 48h		; Register Adressen
TRKREG	EQU 49h		; WD1770
SECREG	EQU 4Ah
DATREG	EQU 4BH
STATUS	EQU COMREG
CTRL	EQU 40H      
;
MOVID	EQU 1003h  	; Warmstart Videomonitor
BREAK	EQU 100Fh	; Unterbrechung mit CTRL-C
PARIN	EQU 1012h	;
STRING	EQU 1021h	; STRING
ERROR	EQU 1027h       ; Error Videomonitor
CLRVID	EQU 102Dh	; CLRVID  
CLRLIN	EQU 1030h	; CLRLIN        
CRSPOS	EQU 1033h	; CRSPOS         
CRSON	EQU 1036h 	; CRSON         
CRSOFF	EQU 1039h	; CRSOFF
BYTOT	EQU 103Fh	; BYTOT  (A) zweistellig darstellen Adresse (HL) 
ADROT	EQU 103ch	; (DE) vierstellig ausgeben
CRLF	EQU 137bh	; gibt CR/LF aus
;
; Exteren Programme
;
FDC	EQU 2000h	; Floppy 
EDIT	EQU 7000h	; Editor
ASS:	EQU 6000h	; Assembler
DISSK	EQU 6C00h	; Disassembler
;
STACK	EQU 2F00H	; 
TRAP	EQU 2F8Ch
RST55	EQU 2F98h
RST551	EQU 2F99h
;
HORDSP	EQU 2E50h	; 40/80 Zeichen pro Zeile
BEGIN	EQU 2E54h	; Anfangsadresse letzte Zeile
UPDATE	EQU 2E56h	; aktuelle Cursorposition
;
SRCBEG	EQU 2FBFh	; Startadresse
SRCEND	EQU 2FC1h	; Endadresse
DSTBEG	EQU 2FC3h	; Anfang Zieladresse/Parameter
DSTEND	EQU 2FC5h	; 
LINZ	EQU 2E54h
;
;uhrbuf:	equ 9800h	; Uhr-BCD
;ubfasc:	equ 982fh 	; Uhr-ASCII Ende des Buffers
;
;uhrbuf:	equ	2fdch	; Uhr-BCD
;ubfasc:	equ	2feah 	; Uhr-ASCII
;
VIDBEG	EQU 3000h	; Bildschirmadresse 	oben links
VIDEND	EQU 377Fh	; 			unten rechts
PRTFLG	EQU 37AFh	; 
RAMBEG	EQU 37B0h	; Anfang Parameterbereich Videomonitor

;
; Buffer Adressen
;
BATBUF	EQU 2D00h
FDCBUF	EQU 2F70h
CPYBUF	EQU 8000h
;
	ORG FDCBUF

TRKNR:	DS 1h		; Spurnummer
SECNR:  DS 1h		; Sektornummer
SIDNR:	DS 1h		; Seite 0/1
SELBYT: DS 1h		; SelectByte Bit1,2 = Laufwerksnummer
SELOUT: DS 1h
TRACK0: DS 4h
RWBEF:	DS 1h
DATADR:	DS 2h		; Buffer-Adresse r/w Sektoren
INTPRG:	DS 2h
RWSUB:	DS 1h
RWSUB1: DS 2h
HIBUF:	DS 2h
BUFDST: DS 1h
BUFSRC: DS 1h		; IM CF-Betrieb = Anzahl der Sektoren die übertragen werden
CPYEND: DS 1h
;
TIMOUT	EQU 32h		;
FQUARZ	EQU 06h		;
DLY100	EQU 13h		;
SECZAL	EQU 16		;
SECLDD	EQU 1		; 256 Bytes
;INTON	EQU 1Eh		; RST 5.5 freigeben
;INTOF	EQU 1Fh		; RST 5.5 sperren
;
	org 2f8ch	; Sprungtabelle fr IRQ- Serviceroutinen

intrap:	ds	3h	; Software IRQ
int3:	ds	3h	;
int4:	ds	3h	;
int5:	ds	3h	;
int55:	ds	3h	; Hardware IRQ
int6:	ds	3h	;
int65:	ds	3h	; Hardware IRQ
int7:	ds	3h	;
int75:	ds	3h	; Hardware IRQ
	
;
;--------------------------------------------------------------------
;
;	Testroutinen Datei mit Hilfe von ISR einlesen
;
;	
	org 	8000h		; Testadresse
	
ust	equ 	9000h		; Userstack	
buffer	equ 	9000h		; Ziel Buffer fuer BIN-Datei
zbuff	equ 	8400h		; Zeilenbuffer
maxbuf	equ 	3fh		; Bufferlaenge 
rtsbuf	equ 	28h 		; Bufferposition fuer RTS
baud	equ 	0040h		; fuer 4800Bd
;
timeout	equ	0600h		; ca. 2s warten
;	
; Sprungtabelle
;
kstart:	jmp 	kalt		; INI
	jmp	rxzirq		; Zeilen mit IRQ lesen
	jmp	0000h		;
;	
;
	org 8020h
; 
; Zwischenspeicher fuer Buffer(FIFO) Steuerung
; (der besseren Uebersicht hier hin verlegt)
;
buffz	dw	0000h		; Schreibzeiger fuer Zielbuffer
wrz	dw	0000h		; Schreibzeiger fuer IRQ
rdz	dw	0000h		; Lesezeiger	
v24flag	db	00h		; 1000 0001
				; :       :
				; :       +-- RTS Punkt erreicht
				; +---------- Bufferueberlauf --> Fehler
;
;
	org	8040h		;
;	
kalt:	lxi	sp,ust		; User Stack setzen
	call 	mioini		; IO-Bausteine initialisieren 
	lxi  	h,meld1		;
	call 	string		;
	jmp 	ende		; 
;
ende: 	lxi  	h,meld10	;
	call 	string		;
	jmp  	movid		; zurueck zum Monitor		
;
;----------------------------------------------------------
; Karte initialisieren
;		
mioini:	mvi	a,bap1		; PIO-Betriebsarten einstellen
	out	pio1s		;
	mvi	a,bap2		;
	out	pio2s		;
	xra	a		;
	out	pio2b		;
	out	pio2c		;
	;
	mvi	a,03h		; Reset com1 1
	out	com1s		;	
	mvi	a,bacom1	; com1-Betriebsart
	out	com1s		;
	mvi	a,55h		; RST On
	out	com1s		;
	;
	mvi	a,03h		; Reset com2 2
	out	com2s		;	
	mvi	a,bacom2	; com2-Betriebsart
	out	com2s		;
	mvi	a,55h		; RST On
	out	com2s		;	
	;
	mvi	a,bati0		; Timer-Betriebsarten
	out	tims		;
	mvi	a,bati1		;
	out	tims		;
	mvi	a,bati2		;
	out	tims		;
	;
	lxi	d,baud		; siehe oben
	mov	a,e		; in Timer 0 laden
	out	tim0		;
	mov	a,d		;
	out	tim0		;
	;
	mov	a,e		; in Timer 1 laden
	out	tim1		;
	mov	a,d		;
	out	tim1		;
	;
	mov	a,e		; in Timer 2 laden
	out	tim2		;
	mov	a,d		;
	out	tim2		;
	;
	ret	
;
;
;====================================================================
;
; V24 mit IRQ Routine - Test fuer Intel-Hexlader
; Daten zeilenweise empfangen und im Buffer ablegen
; 
; Parameter zur Ubersicht
;
; buffer	equ 9000h	; fuer BIN-Datei
; zbuff		equ 8400h	; Buffer (FIFO)
;
;            v-Schreibzeiger (wrz)
;            I
; 8400h I------------...------------------+-...--I 843fh (maxbuf)
;         ^                               :
;         I- Lesezeiger (rdz)       842ah +------ RTS Setzen (rtsbuf)
;
; maxbuf	equ 3fh		; Bufferlaenge 
; rtsbuf	equ 2ah		; Bufferposition fuer RTS
;
; buffz		dw	0000h	; Schreibzeiger fuer Zielbuffer
; wrz		dw	0000h	; Schreibzeiger fuer IRQ
; rdz		dw	0000h	; Lesezeiger	
; fuelst	db	00h	; Anzahl byte im Buffer = Fuellstand
; v24flag	db	00h	; 1000 0001
;				; :       :
;				; :       +-- RTS punkt erreicht
;				; +---------- Buffer voll
;
;--------------------------------------------------------------------
;
rxzirq:	lxi	sp,ust		; User Stack setzen
	lxi  	h,meld11	; Startmeldung
	call 	string		;
	lxi	h,buffer	; Schreib/Lesezeiger
	shld	buffz		;
	lxi	h,zbuff		;  
	shld	wrz		;
	shld	rdz		; auf Anfang
	xra	a		;
	sta	v24flag		; Testzaehler loeschen
	call	mioini		; Karte initialisieren
	call	irqi		; IRQ Serviceroutinen einrichten
	mvi	a,95h		; 1001 0101 = RX IRQ und
	out	com1s		; RTS freigeben
	ei			;
;
; Haupschleife
;
; Haupschleife, wartet auf Daten bis Timeout
;
rxz_l:	push	b		;
rxz_l1	lxi	b,timeout	; Abbruchbedingung laden
rxz_l2:	di			;
	lhld	wrz		;
	xchg			;
	lhld	rdz		;
	mov	a,l		;
	cmp	e		;
	jc	rxzrb		; weiter, Daten lesen
	lda	v24flag		;
	ana	a		;
	jnz	rtscl		; RTS und IRQ freigeben
	ei
	cpi	081h		;
	jz	rxz_e		; Ende mit Bufferueberlauf
	call	dely1		;
	dcr	c		; Timeout ?
	jnz	rxz_l2		;
	dcr	b		;	
	jnz  	rxz_l2		; warten
	
rxz_e:	pop	b		;	
	mvi	a,1ah		;
	di			;
	jmp	ende		; Tiomeout keine Daten (A) = 1Ah
;
; Buffer auslesen
;
rxzrb:	di
	mov	a,m		; Byte holen
	inx	h		;
	shld	rdz		;
	lhld	buffz		;
	mov	m,a		; Byte in Zielbuffer
	inx	h		;
	shld	buffz		;
	ei
	jmp	rxz_l1		; warten auf naechstes Byte

;
; RTS und IRQ freigeben
;
rtscl:	di			;
	lxi	h,zbuff		; Buffer 
	shld	wrz		;
	shld	rdz		; auf Anfang
	xra	a		;
	sta	v24flag		; Testzaehler loeschen
	mvi	a,95h		; 1001 0101 = RX IRQ und
	out	com1s		; RTS freigeben
	ei
	jmp	rxz_l1		;
;
;
;--------------------------------------------------------------------
;
; IRQ vorbereiten
;
irqi:	lxi	h,int65		; Spungtabelle
	mvi	m,0c3h		; = JMP
	inx	h		; IRQ Seviceroutine
	lxi	d,irqs		; In Sprungtabelle
	mov	m,e		;
	inx	h		;
	mov	m,d		; eintragen
	mvi	a,1dh		; irq6.5 freigeben
	sim			;
	ret			;
;
; IRQ Serviceroutine
; Ein Byte uber V24 empfangen und im Empfangsbuffer ablegen
;
irqs: 	di			; IRQ sperren 
	push	psw		; alles sichern
	push	h		;
	push	d		;
	push	b		;
	in	com1s		;
	ani	01h		; Senderegister voll
	jz	irqe		; nein	
	lhld	wrz		; Zeiger auf Zeilenbuffer	
	in	com1d		;
	mov	m,a		; daten ablegen
	inx	h		;
	shld	wrz		; Zeiger sichern
	mvi	a,rtsbuf	; Zeiger an RTS Grenze
	cmp	l		;
	jz	setrts		; RTS Setzen
	mvi	a,maxbuf	; Test auf Ende Zeilenbuffer
	cmp	l		;
	jz	irqbs		; Zeilenbuffer Ende erreicht
	
irqe:	pop	b		; alles wieder zurueck
	pop	d		;
	pop	h		;
	pop	psw		;
	ei			; IRQ freigeben
	ret			;
	
setrts:	mvi	a,0d5h		; RTS gesetzt
	out	com1s		; IRQ Freigabe
	mvi	a,01h		;
	sta	v24flag		;
	jmp	irqe		;

irqbs:	lxi	h,zbuff		; Buffer wieder auf Anfang 
	shld	wrz		;
	mvi	a,55h		; RTS gesetzt
	out	com1s		; IRQ gesperrt
	mvi	a,81h		;
	sta	v24flag		;
	jmp	irqe		;	
;
;====================================================================
;
; Texte
;
meld1:	DB 0Dh			; CR
	DB 0Ah			; LF     
	DB "Multi-IO-Test"
	DB " V0.11 INI"
	DB 0Dh			; 
	DB 00h
;

meld10:	DB 0Dh			; CR
	DB 0Ah			; LF     
	DB "Ende"
	DB 0Dh			;
	DB 00h	 
;	
meld11:	DB 0Dh			; CR
	DB 0Ah			; LF
	DB "Multi-IO-Test"
	DB " V0.11 "	
	DB "Datei lesen"
	DB 0Dh			;
	DB 00h	 
;
;--------------------------------------------------------------------

	